# © 2024 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import Command, api, fields, models


class FinancialContractGuarantee(models.Model):
    _name = "financial.contract.guarantee"
    _inherit = [
        "financial.contract",
        "mail.thread",
        "mail.activity.mixin",
    ]
    _description = "Financial contract guarantee"
    _check_company_auto = True
    _rec_names_search = [
        "name",
        "number",
        "partner_id.name",
        "partner_id.member_number",
    ]
    _order = "create_date desc"

    product_id = fields.Many2one(
        comodel_name="financial.product.template.guarantee",
        string="Gamme",
        required=True,
    )
    suspensive_condition_ids = fields.One2many(
        comodel_name="financial.contract.guarantee.suspensive.condition",
        inverse_name="contract_id",
        compute="_compute_suspensive_condition_ids",
        store=True,
        readonly=False,
    )
    # TODO: voir pour la gestion de ce champ avec le module contract external
    external_loan_id = fields.Many2one(
        comodel_name="financial.contract.external",
        string="Prêt lié",
        domain="[('partner_id', '=', partner_id), ('tool', '=', 'loan')]",
    )
    loan_contract_number = fields.Char(
        string="Numéro de contrat de prêt",
        related="external_loan_id.number",
        store=True,
    )

    is_counter_guarantee = fields.Boolean("Contre Garantie", default=False)
    fmg_amount = fields.Monetary("Montant FMG")
    social_share_amount = fields.Monetary("Montant parts sociales")
    final_risk_guarantee = fields.Monetary(
        string="Garantie risque final",
        compute="_compute_final_risk_guarantee",
        store=True,
        tracking=1,
    )

    line_ids = fields.One2many(
        comodel_name="financial.contract.guarantee.line",
        inverse_name="guarantee_id",
        string="Lignes de solde",
    )

    # loan_balance = fields.Monetary(
    #     string="Solde créance",
    #     compute="_compute_balance",
    #     store=True,
    #     tracking=2,
    # )
    # guarantee_balance = fields.Monetary(
    #     string="Solde Garantie",
    #     compute="_compute_balance",
    #     store=True,
    #     tracking=2,
    # )

    # Loan Data
    payment_date = fields.Date("Date de versement")
    amount_initial = fields.Monetary("Montant initial")
    amount_received = fields.Monetary("Montant reçu")
    end_date = fields.Date("Date de fin de prêt")
    remaining_capital = fields.Monetary("Capital restant dû")
    loan_duration = fields.Integer("Durée du crédit")
    guarantee_rate = fields.Float("Quotité garantie")
    guarantee_amount = fields.Monetary("En cours garanti")
    # Company Data
    segment_code = fields.Char("Code Segment")
    bdf_scoring = fields.Char(
        string="Cotation BDF",
        compute="_compute_guarantee_data",
        store=True,
        readonly=False,
        tracking=2,
    )
    bdf_date = fields.Char("Date Cotation BDF")
    mcdo_scoring = fields.Char("Cotation MacDonough")
    mcdo_date = fields.Date("Date Cotation MacDonough")
    branch_code = fields.Char("Code branche")

    # Invoice Data
    commission_amount = fields.Monetary(
        string="Montant de commission",
        compute="_compute_commission_amount",
        store=True,
    )

    # ------------------------------------------------------
    # Constrains functions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends(
        "line_ids",
        "line_ids.bdf_scoring",
    )
    def _compute_guarantee_data(self):
        for guarantee in self:
            last_line = guarantee.line_ids.sorted("line_date desc")
            guarantee.update({"bdf_scoring": last_line.bdf_scoring})

    @api.depends(
        "line_ids",
        "line_ids.commission_amount",
    )
    def _compute_guarantee_data(self):
        for guarantee in self:
            guarantee.commission_amount = sum(
                guarantee.line_ids.mapped("commission_amount")
            )

    # @api.depends()
    # def _compute_balance(self):
    #     for guarantee in self:
    #         guarantee.loan_balance = 0
    #         guarantee.guarantee_balance = 0

    @api.depends()
    def _compute_final_risk_guarantee(self):
        for guarantee in self:
            guarantee.final_risk_guarantee = 0

    @api.depends("product_id")
    def _compute_suspensive_condition_ids(self):
        for contract in self:
            condition_to_create = []
            if contract.product_id:
                for condition in self.product_id.suspensive_condition_ids:
                    condition_to_create.append(
                        Command.create(
                            {
                                "condition_id": condition.id,
                                "condition_comment": condition.condition_comment,
                            },
                        )
                    )
                if condition_to_create:
                    contract.suspensive_condition_ids.unlink()
                    contract.suspensive_condition_ids = condition_to_create

    # ------------------------------------------------------
    # Onchange
    # ------------------------------------------------------
    @api.onchange("product_id")
    def _onchange_product_id(self):
        self._set_product_values()

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD (Override ORM)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business functions
    # ------------------------------------------------------
    def _set_product_values(self):
        if self.product_id:
            self.guarantee_rate = self.product_id.guarantee_rate
